==> dxirc_1.30.0-1.tar.gz: BUILD/qt/widgets/colorbutton.cpp <==
/*
 *      colorbutton.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "colorbutton.h"
#include <QApplication>
#include <QColorDialog>
#include <QPainter>
#include <QStyleOptionButton>

ColorButton::ColorButton(const QColor &color, const QString &text, QWidget *parent)
: QAbstractButton(parent), m_color(color)
{
    setText(text);
    connect(this, SIGNAL(clicked()), SLOT(changeColor()));
}

QSize ColorButton::sizeHint() const
{
    int height = 16;
    int width = 18;
    if(!text().isEmpty())
    {
        QFontMetrics fm = fontMetrics();
        QSize sz = fm.size(Qt::TextShowMnemonic, text());
        height = qMax(height, sz.height()+16);
        width += 6 + sz.width();
    }
    return QSize(width, height);
}

QSize ColorButton::minimumSizeHint() const
{
    return sizeHint();
}

void ColorButton::paintEvent(QPaintEvent*)
{
    QPainter p(this);
    drawButton(&p);
}

void ColorButton::drawButton(QPainter *p)
{
    QStyleOptionButton buttonOptions;
    buttonOptions.init(this);
    buttonOptions.features = QStyleOptionButton::None;
    buttonOptions.rect = rect();
    buttonOptions.palette = palette();
    buttonOptions.state = (isDown() ? QStyle::State_Sunken : QStyle::State_Raised);
    style()->drawPrimitive(QStyle::PE_PanelButtonBevel, &buttonOptions, p, this);
    p->save();
    drawButtonContent(p);
    p->restore();
    QStyleOptionFocusRect frectOptions;
    frectOptions.init(this);
    frectOptions.rect = style()->subElementRect(QStyle::SE_PushButtonFocusRect, &buttonOptions, this);
    if(hasFocus())
        style()->drawPrimitive(QStyle::PE_FrameFocusRect, &frectOptions, p, this);
}

void ColorButton::drawButtonContent(QPainter *p)
{
    QPalette::ColorGroup cg = (isEnabled() ? (hasFocus() ? QPalette::Active : QPalette::Inactive) : QPalette::Disabled);
    p->setPen(palette().color(cg, QPalette::ButtonText));
    p->setBrush(m_color);
    p->drawRect(4, (height()-12)/2, 12, 12);
    if(!text().isEmpty())
    {
        QFontMetrics fm = fontMetrics();
        QSize sz = fm.size(Qt::TextShowMnemonic, text());
        p->drawText(20, (height()+sz.height()/2)/2, text());
    }
}

void ColorButton::changeColor()
{
    QColor c = QColorDialog::getColor(m_color, qApp->activeWindow());
    if(c.isValid())
    {
        m_color = c;
        emit colorChanged(getColor());
    }
}

==> dxirc_1.40.1-1.tar.gz: BUILD/qt/widgets/colorbutton.cpp <==
/*
 *      colorbutton.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "colorbutton.h"
#include <QApplication>
#include <QColorDialog>
#include <QPainter>
#include <QStyleOptionButton>

ColorButton::ColorButton(const QColor &color, const QString &text, QWidget *parent)
: QAbstractButton(parent), m_color(color)
{
    setText(text);
    connect(this, SIGNAL(clicked()), SLOT(changeColor()));
}

QSize ColorButton::sizeHint() const
{
    int height = 16;
    int width = 18;
    if(!text().isEmpty())
    {
        QFontMetrics fm = fontMetrics();
        QSize sz = fm.size(Qt::TextShowMnemonic, text());
        height = qMax(height, sz.height()+16);
        width += 6 + sz.width();
    }
    return QSize(width, height);
}

QSize ColorButton::minimumSizeHint() const
{
    return sizeHint();
}

void ColorButton::paintEvent(QPaintEvent*)
{
    QPainter p(this);
    drawButton(&p);
}

void ColorButton::drawButton(QPainter *p)
{
    QStyleOptionButton buttonOptions;
    buttonOptions.initFrom(this);
    buttonOptions.features = QStyleOptionButton::None;
    buttonOptions.rect = rect();
    buttonOptions.palette = palette();
    buttonOptions.state = (isDown() ? QStyle::State_Sunken : QStyle::State_Raised);
    style()->drawPrimitive(QStyle::PE_PanelButtonBevel, &buttonOptions, p, this);
    p->save();
    drawButtonContent(p);
    p->restore();
    QStyleOptionFocusRect frectOptions;
    frectOptions.initFrom(this);
    frectOptions.rect = style()->subElementRect(QStyle::SE_PushButtonFocusRect, &buttonOptions, this);
    if(hasFocus())
        style()->drawPrimitive(QStyle::PE_FrameFocusRect, &frectOptions, p, this);
}

void ColorButton::drawButtonContent(QPainter *p)
{
    QPalette::ColorGroup cg = (isEnabled() ? (hasFocus() ? QPalette::Active : QPalette::Inactive) : QPalette::Disabled);
    p->setPen(palette().color(cg, QPalette::ButtonText));
    p->setBrush(m_color);
    p->drawRect(4, (height()-12)/2, 12, 12);
    if(!text().isEmpty())
    {
        QFontMetrics fm = fontMetrics();
        QSize sz = fm.size(Qt::TextShowMnemonic, text());
        p->drawText(20, (height()+sz.height()/2)/2, text());
    }
}

void ColorButton::changeColor()
{
    QColor c = QColorDialog::getColor(m_color, qApp->activeWindow());
    if(c.isValid())
    {
        m_color = c;
        emit colorChanged(getColor());
    }
}
